# XES TAGS
TAG_BOOLEAN = 'boolean'
TAG_CLASSIFIER = 'classifier'
TAG_DATE = 'date'
TAG_EVENT = 'event'
TAG_EXTENSION = 'extension'
TAG_FLOAT = 'float'
TAG_GLOBAL = 'global'
TAG_ID = 'id'
TAG_INT = 'int'
TAG_LIST = 'list'
TAG_LOG = 'log'
TAG_STRING = 'string'
TAG_TRACE = 'trace'
TAG_VALUES = 'values'

# XES/INTERNAL KEYS
KEY_CHILDREN = 'children'
KEY_KEY = 'key'
KEY_KEYS = 'keys'
KEY_NAME = 'name'
KEY_PREFIX = 'prefix'
KEY_SCOPE = 'scope'
KEY_URI = 'uri'
KEY_VALUE = 'value'

DEFAULT_NAME_KEY = 'concept:name'
DEFAULT_TIMESTAMP_KEY = 'time:timestamp'
DEFAULT_START_TIMESTAMP_KEY = 'start_timestamp'
DEFAULT_TRACEID_KEY = 'concept:name'
DEFAULT_RESOURCE_KEY = 'org:resource'
DEFAULT_TRANSITION_KEY = "lifecycle:transition"
