from enum import Enum


class Parameters(Enum):
    SEP = "sep"
    QUOTECHAR = "quotechar"
    NROWS = "nrows"
    SORT = "sort"
    SORT_FIELD = "sort_field"
    TIMEST_FORMAT = "timest_format"
    TIMEST_COLUMNS = "timest_columns"
    INSERT_EVENT_INDEXES = "insert_event_indexes"
    ENCODING = "encoding"
