TO_TRACE_LOG = 'to_event_log'
TO_EVENT_LOG = 'to_event_log'
TO_EVENT_STREAM = 'to_event_stream'
TO_DATAFRAME = 'to_dataframe'
FROM_DATAFRAME = 'from_dataframe'

DF_TO_EVENT_LOG_1V = 'df_to_event_log_1v'
DF_TO_EVENT_LOG_NV = 'df_to_event_log_nv'

DEEPCOPY = "deepcopy"
STREAM_POSTPROCESSING = "stream_postprocessing"
